@@ -21,7 +21,7 @@ resetGlobalVariable()
 # -READ.ONLY please ignore
 # -LOCAL please ignore
 # -ADD please ignore
-# 
+#
 # == details
 # Global parameters for the circular layout. Currently supported parameters are:
 #
@@ -51,9 +51,14 @@ resetGlobalVariable()
 #     `convert_height` can be used to set the height to an absolute unit.
 # -``points.overflow.warning`` Since each cell is in fact not a real plotting region but only
 #     an ordinary rectangle, it does not eliminate points that are plotted out of
-#     the region. So if some points are out of the plotting region, ``circlize`` would continue drawing the points and printing warnings. In some 
+#     the region. So if some points are out of the plotting region, ``circlize`` would continue drawing the points and printing warnings. In some
 #     cases, draw something out of the plotting region is useful, such as draw
 #     some legend or text. Set this value to ``FALSE`` to turn off the warnings.
+# -``circle.margin``  Margin in the horizontal and vertical direction. The value should be a positive numeric vector
+#    and the length of it should be either 1, 2, or 4. When it has length of 1, it controls the margin on the four sides of the circle.
+#    When it has length of 2, the first value controls the margin on the left and right, and the second value controls
+#    the margin on the bottom and top side. When it has length of 4, the four values controls the margins on the left, right, bottom and top sides
+#    of the circle. So A value of ``c(x1, x2, y1, y2)`` means ``circos.par(canvas.xlim = c(-(1+x1), 1+x2), canvas.ylim = c(-(1+y1), 1+y2))``.
 # -``canvas.xlim``              The coordinate for the canvas. Because ``circlize`` draws everything (or almost everything) inside the unit circle,
 #     the default ``canvas.xlim`` and ``canvas.ylim`` for the canvas would be all ``c(-1, 1)``. However, you can set it to a more broad
 #     interval if you want to draw other things out of the circle. By choosing proper
@@ -62,17 +67,24 @@ resetGlobalVariable()
 #     circle in the region of (0, pi/2).
 # -``canvas.ylim``              The coordinate for the canvas. By default it is ``c(-1, 1)``
 # -``clock.wise``               The direction for adding sectors. Default is ``TRUE``.
+# -``xaxis.clock.wise``    The direction in the x-axes for all sectors. Default is ``TRUE``.
 #
-# Similar as `graphics::par`, you can get the parameter values by specifying the 
+# Similar as `graphics::par`, you can get the parameter values by specifying the
 # names of parameters and you can set the parameter values by specifying a
 # named list which contains the new values.
 #
-# ``gap.degree``, ``start.degree``, ``canvas.xlim``, ``canvas.ylim`` and ``clock.wise`` 
+# ``gap.degree``, ``start.degree``, ``canvas.xlim``, ``canvas.ylim`` and ``clock.wise``
 # only be set before the initialization of the circular layout
 # (i.e. before calling `circos.initialize`) because these values will not be changed after
 # adding sectors on the circle. The left and right padding for ``cell.padding`` will also be
 # ignored after the initialization because all cells in a sector would share the same
-# left and right paddings. 
+# left and right paddings.
+#
+# == seealso
+# https://jokergoo.github.io/circlize_book/book/circular-layout.html#graphic-parameters
+#
+# == example
+# circos.par
 circos.par = function(..., RESET = FALSE, READ.ONLY = NULL, LOCAL = FALSE, ADD = FALSE) {}
 circos.par = setGlobalOptions(
 	start.degree = list(
@@ -127,8 +139,31 @@ circos.par = setGlobalOptions(
 		}),
 	track.height = 0.2,
 	points.overflow.warning = TRUE,
+	circle.margin = list(
+		.value = c(0, 0, 0, 0),
+		.filter = function(x) {
+			if(is.circos.initialized()){
+				warning_wrap("'circle.margin' can only be modified before `circos.initialize`, or maybe you forgot to call `circos.clear` in your last plot.")
+			}
+			if(any(x <= 0)) {
+				stop_wrap("The value of `circle.margin` should be positive.")
+			}
+			if(length(x) == 1) {
+				x = rep(x, 4)
+			} else if(length(x) == 2) {
+				x = rep(x, each = 2)
+			} else if(length(x) == 4) {
+
+			} else {
+				stop_wrap("Length of `circle.margin` can only be 1, 2, or 4.")
+			}
+			return(x)
+		}
+	),
 	canvas.xlim = list(
-		.value = c(-1, 1),
+		.value = function() {
+			c(-(1 + .v$circle.margin[1]), (1 + .v$circle.margin[2]))
+		},
 		.filter = function(x) {
 			if(is.circos.initialized()){
 				warning_wrap("'canvas.xlim' can only be modified before `circos.initialize`, or maybe you forgot to call `circos.clear` in your last plot.")
@@ -136,7 +171,9 @@ circos.par = setGlobalOptions(
 			return(x)
 		}),
 	canvas.ylim = list(
-		.value = c(-1, 1),
+		.value = function() {
+			c(-(1 + .v$circle.margin[3]), (1 + .v$circle.margin[4]))
+		},
 		.filter = function(x) {
 			if(is.circos.initialized()){
 				warning_wrap("'canvas.ylim' can only be modified before `circos.initialize`, or maybe you forgot to call `circos.clear` in your last plot.")
@@ -152,6 +189,14 @@ circos.par = setGlobalOptions(
 			}
 			return(x)
 		}),
+	xaxis.clock.wise = list(
+		.value = TRUE,
+		.filter = function(x) {
+			if(is.circos.initialized()){
+				stop_wrap("'xaxis.clock.wise' can only be modified before `circos.initialize`, or maybe you forgot to call `circos.clear` in your last plot.")
+			}
+			return(x)
+		}),
 	lend = list(
 		.value = NULL,
 		.visible = FALSE,
@@ -168,7 +213,13 @@ circos.par = setGlobalOptions(
 	'__omar__' = list(   # is par("mar") the default value?
 		.value = FALSE,
 		.private = TRUE,
-		.visible = FALSE)
+		.visible = FALSE),
+	'__tempenv__' = list(
+		.value = new.env(parent = emptyenv()),
+		.private = TRUE,
+		.visible = FALSE),
+	message = TRUE,
+	help = list(.synonymous = "message")
 )
 
 # before initialization, .SECTOR.DATA is NULL
@@ -181,12 +232,13 @@ is.circos.initialized = function() {
 # Initialize the circular layout
 #
 # == param
-# -factors A `factor` variable or a character vector which represent data categories
+# -sectors A `factor` variable or a character vector which represent data categories
+# -factors The same as ``sectors``. It will be removed in future versions. 
 # -x       Data on x-axes, a vector
 # -xlim    Ranges for values on x-axes, see "details" section for explanation of the format
 # -sector.width Width for each sector. The length of the vector should be either 1 which means
 #          all sectors have same width or as same as the number of sectors. Values for
-#          the vector are relative, and they will be scaled by dividing their summation. 
+#          the vector are relative, and they will be scaled by dividing their summation.
 #          By default, it is ``NULL`` which means the width of sectors correspond to the data
 #          range in sectors.
 #
@@ -215,18 +267,39 @@ is.circos.initialized = function() {
 # The function finally calls `graphics::plot` with enforing aspect ratio to be 1 and be ready for adding graphics.
 #
 # == seealso
-# http://jokergoo.github.io/circlize_book/book/circular-layout.html
-circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL) {
+# https://jokergoo.github.io/circlize_book/book/circular-layout.html
+circos.initialize = function(
+	sectors = NULL, 
+	x = NULL,
+	xlim = NULL,
+	sector.width = NULL,
+	factors = sectors) {
 
     resetGlobalVariable()
-	
+
 	.SECTOR.DATA = get(".SECTOR.DATA", envir = .CIRCOS.ENV)
 	.CELL.DATA = get(".CELL.DATA", envir = .CIRCOS.ENV)
-	
-	if(any(factors == "")) {
-		stop_wrap("`factors` cannot contain empty strings.")
+
+	if(is.null(factors)) {
+		if(is.matrix(xlim) || is.data.frame(xlim)) {
+			if(is.null(rownames(xlim))) {
+				stop_wrap("Since `sectors` is not specified, row names of `xlim` are taken as `sectors`, thus `xlim` should be a two-column matrix with row names.")
+			} else {
+				factors = rownames(xlim)
+			}
+		} else {
+			stop_wrap("Since `sectors` is not specified, row names of `xlim` are taken as `sectors`, thus `xlim` should be a two-column matrix with row names.")
+		}
 	}
-	
+
+	if(is.numeric(factor)) {
+		warning_wrap("Your `sectors` is numeric, it will be converted to characters internally.")
+	}
+
+	if(any(factors == "")) {
+		stop_wrap("`sectors` cannot contain empty strings.")
+	}
+
     if(! is.factor(factors)) {
         if(length(factors) == length(unique(factors))) {
            factors = factor(factors, levels = factors)
@@ -240,7 +313,7 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
     if(!is.null(x)) {
     	x = as.numeric(x)
     }
-    
+
     # initialize .SECTOR.DATA
     # you can think it as the global x axis configuration
     # calculate min and max value for each sectors
@@ -249,15 +322,15 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
     if(is.vector(xlim)) {
         if(length(xlim) != 2) {
             stop_wrap("Since `xlim` is vector, it should have length of 2.")
-        }    
+        }
 
         xlim = as.numeric(xlim)
-        
+
         min.value = rep(xlim[1], length(le))
         max.value = rep(xlim[2], length(le))
     } else if(is.matrix(xlim) || is.data.frame(xlim)) {
         if(dim(xlim)[1] != length(le) || dim(xlim)[2] != 2) {
-            stop_wrap("Since `xlim` is a matrix, it should have same number of rows as the length of the level of `factors` and number of columns of 2.")
+            stop_wrap("Since `xlim` is a matrix, it should have same number of rows as the length of the level of `sectors` and number of columns of 2.")
         }
 
         if(!is.null(rownames(xlim))) {
@@ -270,67 +343,73 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
     	dim(xlim2) = dim(xlim)
     	dimnames(xlim2) = dimnames(xlim)
     	xlim = xlim2
-        
+
         min.value = apply(xlim, 1, function(x) x[1])
         max.value = apply(xlim, 1, function(x) x[2])
     } else if(is.vector(x)) {
-    
+
         if(length(x) != length(factors)) {
-            stop_wrap("Length of `x` and length of `factors` differ.")
+            stop_wrap("Length of `x` and length of `sectors` differ.")
         }
         min.value = tapply(x, factors, min)
         max.value = tapply(x, factors, max)
     } else {
 		stop_wrap("You should specify either `x` or `xlim`.")
 	}
-    
+
     cell.padding = circos.par("cell.padding")
-    
+
 	# range for sectors
     sector.range = max.value - min.value
     n.sector = length(le)
-    
+
     sector = vector("list", 7)
 	# for each sector, `start.degree always referto `min.value` and `end.degree` always
-	# refer to `max.value` in a reverse clockwise fasion. So here `start.degree` and 
+	# refer to `max.value` in a reverse clockwise fasion. So here `start.degree` and
 	# `end.degree` also correspond to the direction.
 	# So in the polar coordinate, `start.degree` would be larger than `end.degree`
     names(sector) = c("factor", "min.value", "max.value", "start.degree", "end.degree", "min.data", "max.data")
     sector[["factor"]] = le
 	sector[["min.data"]] = min.value
 	sector[["max.data"]] = max.value
-    
+
     gap.degree = circos.par("gap.degree")
+    if(!is.null(names(gap.degree))) {
+    	if(length(setdiff(le, names(gap.degree))) == 0) {
+    		gap.degree = gap.degree[le]
+    	}
+    }
+
 	if(length(gap.degree) == 1) {
 		gap.degree = rep(gap.degree, n.sector)
 	} else if(length(gap.degree) != n.sector) {
-		stop_wrap("Since `gap.degree` parameter has length larger than 1, it should have same length as number of levels of factors.")
+		stop_wrap("Since `gap.degree` parameter has length larger than 1, it should have same length as the number of sectors.")
 	}
-	
+
 	start.degree = circos.par("start.degree")
 	clock.wise = circos.par("clock.wise")
-    
+
     if(360 - sum(gap.degree) <= 0) {
 		stop_wrap("Maybe your `gap.degree` is too large so that there is no space to allocate sectors.")
 	}
-		
+
     if(is.null(sector.width)) {
 		# degree per data
 		unit = (360 - sum(gap.degree)) / sum(sector.range)
-		
+
 		for(i in seq_len(n.sector)) {
-			
+
 			if(sector.range[i] == 0) {
-				stop_wrap("Range of the sector ('", le[i] ,"') cannot be 0.")
+				stop_wrap("Range of the sector ('", le[i] ,"') cannot be 0. You might need to set the second and the fourth values in `circos.par$cell.padding` to 0.")
 			}
-			
+
 			# only to ensure value are always increasing or decreasing with the absolute degree value
 			if(clock.wise) {
 				sector[["start.degree"]][i] = ifelse(i == 1, start.degree, sector[["end.degree"]][i-1] - gap.degree[i-1])
 				sector[["end.degree"]][i] =  sector[["start.degree"]][i] - sector.range[i]*unit
 			} else {
 				sector[["end.degree"]][i] = ifelse(i == 1, start.degree, sector[["start.degree"]][i-1] + gap.degree[i-1])
-				sector[["start.degree"]][i] = sector[["end.degree"]][i] + sector.range[i]*unit   
+				sector[["start.degree"]][i] = sector[["end.degree"]][i] + sector.range[i]*unit
 			}
 		}
 	} else {
@@ -339,28 +418,28 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
 		} else if(length(sector.width) != n.sector) {
 			stop_wrap("Since you manually set the width for each sector, the length of `sector.width` should be either 1 or as same as the number of sectors.")
 		}
-		
+
 		sector.width.percentage = sector.width / sum(sector.width)
 		degree.per.sector = (360 - sum(gap.degree)) * sector.width.percentage
-		
+
 		if(any(degree.per.sector <= 0)) {
 			stop_wrap("Maybe your `gap.degree` is too large so that there is no space to allocate sectors.")
 		}
-		
+
 		for(i in seq_len(n.sector)) {
-			
+
 			if(sector.range[i] == 0) {
 				stop_wrap("Range of the sector (", le[i] ,") cannot be 0.")
 			}
-			
-			
+
+
 			# only to ensure value are always increasing or decreasing with the absolute degree value
 			if(clock.wise) {
 				sector[["start.degree"]][i] = ifelse(i == 1, start.degree, sector[["end.degree"]][i-1] - gap.degree[i-1])
 				sector[["end.degree"]][i] =  sector[["start.degree"]][i] - degree.per.sector[i]
 			} else {
 				sector[["end.degree"]][i] = ifelse(i == 1, start.degree, sector[["start.degree"]][i-1] + gap.degree[i-1])
-				sector[["start.degree"]][i] = sector[["end.degree"]][i] + degree.per.sector[i] 
+				sector[["start.degree"]][i] = sector[["end.degree"]][i] + degree.per.sector[i]
 			}
 		}
 	}
@@ -372,32 +451,32 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
 		sector[["start.degree"]] = sector[["start.degree"]] + 360
 		sector[["end.degree"]] = sector[["end.degree"]] + 360
 	}
-	
+
 	if(any(cell.padding[2] + cell.padding[4] >= sector[["start.degree"]] - sector[["end.degree"]])) {
 		stop_wrap("Summation of cell padding on x-direction are larger than the width for some sectors. You can e.g. set 'circos.par(cell.padding = c(0.02, 0, 0.02, 0))' or remove tiny sectors.")
 	}
-	
+
 	min.value = min.value - cell.padding[2]/(sector[["start.degree"]] - sector[["end.degree"]] - cell.padding[2] - cell.padding[4])*sector.range  # real min value
     max.value = max.value + cell.padding[4]/(sector[["start.degree"]] - sector[["end.degree"]] - cell.padding[2] - cell.padding[4])*sector.range  # real max value
     sector[["min.value"]] = min.value
     sector[["max.value"]] = max.value
-    
+
     sector = as.data.frame(sector, stringsAsFactors = FALSE)
     .SECTOR.DATA = sector
-    
+
     # initialize .CELL.DATA which contains information of each cell
-    # if content of that cell has been created, it means that the 
+    # if content of that cell has been created, it means that the
     # plotteing region for that cell has been created.
     .CELL.DATA = vector("list", length = length(le))
     names(.CELL.DATA) = le
     for(i in seq_along(.CELL.DATA)) {
         .CELL.DATA[[ le[i] ]] = vector("list", length = 0)
     }
-	
+
 	assign(".SECTOR.DATA", .SECTOR.DATA, envir = .CIRCOS.ENV)
 	assign(".CELL.DATA", .CELL.DATA, envir = .CIRCOS.ENV)
 	assign(".CURRENT.SECTOR.INDEX", .SECTOR.DATA[1, "factor"], envir = .CIRCOS.ENV)
-    
+
     circos.par("__omar__" = FALSE)
 	if(identical(par("mar"), c(5.1, 4.1, 4.1, 2.1))) {
 		circos.par("__omar__" = TRUE)
@@ -405,7 +484,7 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
 	}
     # draw everything in a unit circle
 	plot(circos.par("canvas.xlim"), circos.par("canvas.ylim"), type = "n", ann = FALSE, axes = FALSE, asp = 1)
-    
+
 	# all the information of cells would be visited through `get.cell.meta.data`
 	return(invisible(NULL))
 }
@@ -420,7 +499,7 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
 #
 # If you meet some errors when re-drawing the circular plot, try running this function and it will solve most of the problems.
 circos.clear = function() {
-    
+
 	resetGlobalVariable()
 	if(circos.par("__omar__")) {
 		circos.par("__omar__" = FALSE)
@@ -430,9 +509,16 @@ circos.clear = function() {
 	circos.par(RESET = TRUE)
 	circos.par("__tempdir__" = tmpdir)
 
+	empty_env(circos.par("__tempenv__"))
+
     return(invisible(NULL))
 }
 
+empty_env = function(env) {
+	obj = ls(envir = env, all.names = TRUE)
+	if(length(obj)) rm(list = obj, envir = env)
+}
+
 # == title
 # Get index for all sectors
 #
@@ -456,12 +542,15 @@ get.all.track.index = function() {
 	.CELL.DATA = get(".CELL.DATA", envir = .CIRCOS.ENV)
 	if(is.null(.CELL.DATA)) {
 		return(integer(0))
+	} else if(all(sapply(.CELL.DATA, length) == 0)) {
+		return(integer(0))
 	} else {
-		return(seq_along(.CELL.DATA[[ which(sapply(.CELL.DATA, length) > 0)[1] ]]))
+		return(seq_len(max(sapply(.CELL.DATA[ which(sapply(.CELL.DATA, length) > 0) ], function(x) length(x)))))
 	}
 }
 
 get.sector.data = function(sector.index = get.current.sector.index()) {
+	sector.index = as.character(sector.index)
 	.SECTOR.DATA = get(".SECTOR.DATA", envir = .CIRCOS.ENV)
     sector.data = as.vector(as.matrix(.SECTOR.DATA[.SECTOR.DATA[[1]] == sector.index, -1]))
     names(sector.data) = colnames(.SECTOR.DATA)[-1]
@@ -475,7 +564,7 @@ get.sector.data = function(sector.index = get.current.sector.index()) {
 # Simply returns the numeric index for the current track.
 get.current.track.index = function() {
 	.CURRENT.TRACK.INDEX = get(".CURRENT.TRACK.INDEX", envir = .CIRCOS.ENV)
-    return(.CURRENT.TRACK.INDEX)   
+    return(.CURRENT.TRACK.INDEX)
 }
 
 set.current.track.index = function(x) {
@@ -491,7 +580,7 @@ set.current.track.index = function(x) {
 # Simply returns the name of current sector
 get.current.sector.index = function() {
 	.CURRENT.SECTOR.INDEX = get(".CURRENT.SECTOR.INDEX", envir = .CIRCOS.ENV)
-    return(.CURRENT.SECTOR.INDEX)   
+    return(.CURRENT.SECTOR.INDEX)
 }
 
 set.current.sector.index = function(x) {
@@ -525,16 +614,20 @@ set.current.sector.index = function(x) {
 # circos.clear()
 # dev.off()
 set.current.cell = function(sector.index, track.index) {
+	sector.index = as.character(sector.index)
 	set.current.sector.index(sector.index)
 	set.current.track.index(track.index)
 }
 
 get.cell.data = function(sector.index = get.current.sector.index(), track.index = get.current.track.index()) {
+	sector.index = as.character(sector.index)
 	.CELL.DATA = get(".CELL.DATA", envir = .CIRCOS.ENV)
     .CELL.DATA[[sector.index]][[track.index]]
 }
 
 set.cell.data = function(sector.index = get.current.sector.index(), track.index = get.current.track.index(), ...) {
+	sector.index = as.character(sector.index)
+	
 	.CELL.DATA = get(".CELL.DATA", envir = .CIRCOS.ENV)
     .CELL.DATA[[sector.index]][[track.index]] = list(...)
 	assign(".CELL.DATA", .CELL.DATA, envir = .CIRCOS.ENV)
@@ -543,7 +636,8 @@ set.cell.data = function(sector.index = get.current.sector.index(), track.index
 
 # whether cell in sector.index, track.index exists?
 has.cell = function(sector.index, track.index) {
-
+	sector.index = as.character(sector.index)
+	
 	.CELL.DATA = get(".CELL.DATA", envir = .CIRCOS.ENV)
     if(sector.index %in% names(.CELL.DATA) &&
        track.index <= length(.CELL.DATA[[sector.index]]) &&
@@ -560,22 +654,25 @@ has.cell = function(sector.index, track.index) {
 # == param
 # -sector.index Which sectors you want to look at? It can be a vector.
 # -track.index  Which tracks you want to look at? It can be a vector.
-# -plot         Whether to add information on the plot
+# -plot         Whether to add information on the plot.
 #
 # == details
 # It tells you the basic parameters for sectors/tracks/cells. If both ``sector.index``
-# and ``track.index`` are set to ``NULL``, the function would print index for 
+# and ``track.index`` are set to ``NULL``, the function would print index for
 # all sectors and all tracks. If ``sector.index`` and/or ``track.index`` are set,
 # the function would print ``xlim``, ``ylim``, ``cell.xlim``, ``cell.ylim``,
-# ``xplot``, ``yplot``, ``track.margin`` and ``cell.padding`` for every cell in specified sectors and tracks.
+# ``xplot``, ``yplot``, ``cell.width``, ``cell.height``, ``track.margin`` and ``cell.padding`` for every cell in specified sectors and tracks.
 # Also, the function will print index of your current sector and current track.
 #
-# If ``plot`` is set to ``TRUE``, the function will plot the index of the sector and the track 
+# If ``plot`` is set to ``TRUE``, the function will plot the index of the sector and the track
 # for each cell on the figure.
+#
+# == seealso
+# https://jokergoo.github.io/circlize_book/book/circular-layout.html#circos-info-and-circos-clear
 circos.info = function(sector.index = NULL, track.index = NULL, plot = FALSE) {
 	sectors = get.all.sector.index()
 	tracks = get.all.track.index()
-		
+
 	if(plot) {
 		for(i in seq_along(sectors)) {
 			for(j in seq_along(tracks)) {
@@ -619,6 +716,8 @@ circos.info = function(sector.index = NULL, track.index = NULL, plot = FALSE) {
 					cell.ylim = get.cell.meta.data("cell.ylim", sector.index[i], track.index[j])
 					xplot = get.cell.meta.data("xplot", sector.index[i], track.index[j])
 					yplot = get.cell.meta.data("yplot", sector.index[i], track.index[j])
+				    cell.width = get.cell.meta.data("cell.width", sector.index[i], track.index[j])
+				    cell.height = get.cell.meta.data("cell.height", sector.index[i], track.index[j])
 				    track.margin = get.cell.meta.data("track.margin", sector.index[i], track.index[j])
 				    cell.padding = get.cell.meta.data("cell.padding", sector.index[i], track.index[j])
 					cat("xlim: [", xlim[1], ", ", xlim[2], "]\n", sep = "")
@@ -627,18 +726,20 @@ circos.info = function(sector.index = NULL, track.index = NULL, plot = FALSE) {
 					cat("cell.ylim: [", cell.ylim[1], ", ", cell.ylim[2], "]\n", sep = "")
 					cat("xplot (degree): [", xplot[1], ", ", xplot[2], "]\n", sep = "")
 					cat("yplot (radius): [", yplot[1], ", ", yplot[2], "]\n", sep = "")
+					cat("cell.width (degree): ", cell.width, "\n", sep = "")
+					cat("cell.height (radius): ", cell.height, "\n", sep = "")
 					cat("track.margin: c(", track.margin[1], ", ", track.margin[2], ")\n", sep = "")
 					cat("cell.padding: c(", cell.padding[1], ", ", cell.padding[2], ", ", cell.padding[3], ", ", cell.padding[4], ")\n", sep = "")
 					cat("\n")
 				}
 			}
-				
+
 		}
 
 		if(length(get.current.sector.index())) cat("Your current sector.index is ", get.current.sector.index(), "\n", sep = "")
 		if(get.current.track.index() > 0) cat("Your current track.index is ", get.current.track.index(), "\n", sep = "")
 	}
-	
+
 }
 
 
@@ -668,14 +769,16 @@ show.index = function() {
 # -``track.index``          Numeric index for the track
 # -``xlim``                 Minimal and maximal values on the x-axis
 # -``ylim``                 Minimal and maximal values on the y-axis
-# -``xrange``               Range of ``xlim``. It equals to ``xlim[2] - xlim[1]`` 
+# -``xrange``               Range of ``xlim``. It equals to ``xlim[2] - xlim[1]``
 # -``yrange``               Range of ``ylim``
-# -``xcenter``              Center of x-axis. It equals to ``(xlim[2] + xlim[1])/2`` 
+# -``xcenter``              Center of x-axis. It equals to ``(xlim[2] + xlim[1])/2``
 # -``ycenter``              Center of y-axis
 # -``cell.xlim``            Minimal and maximal values on the x-axis extended by cell paddings
 # -``cell.ylim``            Minimal and maximal values on the y-axis extended by cell paddings
-# -``xplot``                Degrees for right and left borders of the cell.
+# -``xplot``                Degrees for right and left borders of the cell. The values ignore the direction of the circular layout (i.e. whether it is clock wise or not).
 # -``yplot``                Radius for top and bottom borders of the cell.
+# -``cell.width``           Width of the cell, in degrees.
+# -``cell.height``          Height of the cell, simply ``yplot[2] - yplot[1]``
 # -``cell.start.degree``    Same as ``xplot[1]``
 # -``cell.end.degree``      Same as ``xplot[2]``
 # -``cell.bottom.radius``   Same as ``yplot[1]``
@@ -690,15 +793,18 @@ show.index = function() {
 # `CELL_META` is a short version of `get.cell.meta.data`.
 #
 # == example
-# factors = letters[1:4]
-# circos.initialize(factors, xlim = c(0, 1))
+# sectors = letters[1:4]
+# circos.initialize(sectors, xlim = c(0, 1))
 # circos.trackPlotRegion(ylim = c(0, 1), panel.fun = function(x, y) {
 #     print(get.cell.meta.data("xlim"))
 # })
 # print(get.cell.meta.data("xlim", sector.index = "a", track.index = 1))
 # circos.clear()
-get.cell.meta.data = function(name, sector.index = get.current.sector.index(), 
+get.cell.meta.data = function(name, sector.index = get.current.sector.index(),
                               track.index = get.current.track.index()) {
+	
+	sector.index = as.character(sector.index)
+
 	if(length(sector.index) == 0) {
 		stop_wrap("It seems the circular plot has not been initialized.")
 	}
@@ -724,11 +830,11 @@ get.cell.meta.data = function(name, sector.index = get.current.sector.index(),
 	current.sector.data = get.sector.data(sector.index)
 	current.cell.data = get.cell.data(sector.index, track.index)
 	cell.padding = current.cell.data$cell.padding
-	
+
 	if(length(name) != 1) {
 		stop_wrap("``name`` should only have length of 1.")
 	}
-	
+
 	if(name == "xlim") {
 		return(current.cell.data$xlim)
 	} else if(name == "ylim") {
@@ -762,6 +868,12 @@ get.cell.meta.data = function(name, sector.index = get.current.sector.index(),
 		return(x)
 	} else if(name == "yplot") {
 		return(c(current.cell.data$track.start - current.cell.data$track.height, current.cell.data$track.start))
+	} else if(name == "cell.width") {
+		x = current.sector.data[c("start.degree", "end.degree")]
+		return((x[1] - x[2]) %% 360)
+	} else if(name == "cell.height") {
+		y = c(current.cell.data$track.start - current.cell.data$track.height, current.cell.data$track.start)
+		return(y[2] - y[1])
 	} else if(name == "track.margin") {
 		return(current.cell.data$track.margin)
 	} else if(name == "cell.padding") {
@@ -791,11 +903,43 @@ get.cell.meta.data = function(name, sector.index = get.current.sector.index(),
 	} else if(name == "track.height") {
 		return(current.cell.data$track.height)
 	} else {
-		stop_wrap("Wrong cell meta name.")
+		env = circos.par("__tempenv__")
+		if(!is.null(env$track.meta.data)) {
+			track.index = as.character(track.index)
+			if(!is.null(env$track.meta.data[[track.index]])) {
+				if(name %in% names(env$track.meta.data[[track.index]])) {
+					return(env$track.meta.data[[track.index]][[name]])
+				}
+			}
+		}
+		if(!is.null(env$sector.meta.data)) {
+			if(!is.null(env$sector.meta.data[[sector.index]])) {
+				if(name %in% names(env$sector.meta.data[[sector.index]])) {
+					return(env$sector.meta.data[[sector.index]][[name]])
+				}
+			}
+		}
 	}
+
 	return(NULL)
 }
 
+add.track.meta.data = function(name, value, track.index = get.current.track.index()) {
+	env = circos.par("__tempenv__")
+	if(is.null(env$track.meta.data)) env$track.meta.data = list()
+	track.index = as.character(track.index)
+	if(is.null(env$track.meta.data[[track.index]])) env$track.meta.data[[track.index]] = list()
+	env$track.meta.data[[track.index]][[name]] = value
+}
+
+add.sector.meta.data = function(name, value, sector.index = get.current.sector.index()) {
+	sector.index = as.character(sector.index)
+	env = circos.par("__tempenv__")
+	if(is.null(env$sector.meta.data)) env$sector.meta.data = list()
+	if(is.null(env$sector.meta.data[[sector.index]])) env$sector.meta.data[[sector.index]] = list()
+	env$sector.meta.data[[sector.index]][[name]] = value
+}
+
 # == title (variable:CELL_META)
 # Easy way to get meta data in the current cell
 #
@@ -810,14 +954,15 @@ get.cell.meta.data = function(name, sector.index = get.current.sector.index(),
 # pdf(NULL)
 # circos.initialize("a", xlim = c(0, 1))
 # circos.track(ylim = c(0, 1), panel.fun = function(x, y) {
-# 	print(CELL_META$sector.index)
-# 	print(CELL_META$xlim)
+#     print(CELL_META$sector.index)
+#     print(CELL_META$xlim)
 # })
+# print(names(CELL_META))
 # dev.off()
 CELL_META = "don't use me directly"
 class(CELL_META) = "CELL_META"
 
-# == title 
+# == title
 # Names of all meta data in the current cell
 #
 # == param
@@ -826,10 +971,32 @@ class(CELL_META) = "CELL_META"
 # == example
 # names(CELL_META)
 names.CELL_META = function(x) {
-	c("xlim", "ylim", "xrange", "yrange", "xcenter", "ycenter", "cell.xlim", "cell.ylim",
-     "sector.numeric.index", "sector.index", "track.index", "xplot", "yplot", "track.margin", "cell.padding",
+	sector.index = get.current.sector.index()
+    track.index = get.current.track.index()
+
+	nm = c("xlim", "ylim", "xrange", "yrange", "xcenter", "ycenter", "cell.xlim", "cell.ylim",
+     "sector.numeric.index", "sector.index", "track.index", "xplot", "yplot", "cell.width", "cell.height", "track.margin", "cell.padding",
      "cell.start.degree", "cell.end.degree", "cell.bottom.radius", "cell.top.radius", "bg.col", "bg.border",
      "bg.lty", "bg.lwd", "track.height")
+
+	env = circos.par("__tempenv__")
+	if(track.index > 0) {
+		if(!is.null(env$track.meta.data)) {
+			track.index = as.character(track.index)
+			if(!is.null(env$track.meta.data[[track.index]])) {
+				nm = c(nm, names(env$track.meta.data[[track.index]]))
+			}
+		}
+	}
+	if(!is.null(sector.index)) {
+		if(!is.null(env$sector.meta.data)) {
+			if(!is.null(env$sector.meta.data[[sector.index]])) {
+				nm = c(nm, names(env$sector.meta.data[[sector.index]]))
+			}
+		}
+	}
+
+	return(nm)
 }
 
 # == title
@@ -850,9 +1017,9 @@ names.CELL_META = function(x) {
 }
 
 # == title
-# Print CELL_META 
+# Print CELL_META
 #
-# == param 
+# == param
 # -x input
 # -... additional parameters
 #
